;;; -*-lisp-*-
;;;
;;; $Id$
;;;
;;; Basic system-specific stuff
;;;
;;; (c) 2005 Mark Wooding
;;;

;;;----- Licensing notice ---------------------------------------------------
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;; 
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;; 
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software Foundation,
;;; Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

(defpackage #:runlisp
  (:use #:common-lisp)
  (:export #:*lisp-interpreter* #:*command-line-strings* #:run))
(defvar runlisp:*command-line-strings* '("<interactive>"))

(defpackage #:mdw.sys-base
  (:use #:common-lisp #:runlisp)
  (:export #:exit #:hard-exit #:*program-name* #:*command-line-strings*)
  (:import-from #:runlisp #:*lisp-interpreter* #:*command-line-strings*)
  #+clisp (:import-from #:ext #:exit))
(in-package #:mdw.sys-base)

(defun hard-exit (&optional (code 0))
  "Stops the program immediately in its tracks.  Does nothing else.  Use
   after fork, for example, to avoid flushing buffers."
  (declare (type (unsigned-byte 32) code))
  #+cmu (unix::void-syscall ("_exit" c-call:int) code)
  #+(or clisp ecl) (ext:quit code))

#-clisp
(defun exit (&optional (code 0))
  "Polite way to end a program.  If running in an interactive Lisp, just
   return to the top-level REPL."
  (if (boundp '*lisp-interpreter*)
      #+(or cmu ecl) (ext:quit code)
      (progn
        (unless (zerop code)
          (format t "~&Exiting unsuccessfully with code ~D.~%" code))
        (abort))))

(defvar *program-name*
  (pathname-name (car *command-line-strings*))
  "A plausible guess at the program's name, stripped of strange extensions.")

;;;----- That's all, folks --------------------------------------------------
